/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.customer.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.core.sign.utils.MD5;
import vip.xiaonuo.biz.modular.customer.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.customer.entity.BizCustomer;
import vip.xiaonuo.biz.modular.customer.service.BizCustomerService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 客户控制器
 *
 * @author fanzherong
 * @date  2025/03/21 11:44
 */
@Slf4j
@Tag(name = "客户控制器")
@RestController
@Validated
public class BizCustomerController {

    @Resource
    private BizCustomerService bizCustomerService;

    /**
     * 获取客户分页
     *
     * @author fanzherong
     * @date  2025/03/21 11:44
     */
    @Operation(summary = "获取客户分页")
    @GetMapping("/biz/customer/page")
    public CommonResult<Page<BizCustomer>> page(BizCustomerPageParam bizCustomerPageParam) {
        return CommonResult.data(bizCustomerService.page(bizCustomerPageParam));
    }

    /**
     * 获取客户列表
     *
     * @author sandy
     * @date  2025/03/26 17:00
     */
    @Operation(summary = "获取客户列表")
    @GetMapping("/biz/customer/getList")
    public CommonResult<List<BizCustomer>> getList(BizCustomerPageParam bizCustomerPageParam) {
        return CommonResult.data(bizCustomerService.getList(bizCustomerPageParam));
    }

    /**
     * 添加客户
     *
     * @author fanzherong
     * @date  2025/03/21 11:44
     */
    @Operation(summary = "添加客户")
    @CommonLog("添加客户")
    @SaCheckPermission("/biz/customer/add")
    @PostMapping("/biz/customer/add")
    public CommonResult<String> add(@RequestBody @Valid BizCustomerAddParam bizCustomerAddParam) {
        bizCustomerService.add(bizCustomerAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑客户
     *
     * @author fanzherong
     * @date  2025/03/21 11:44
     */
    @Operation(summary = "编辑客户")
    @CommonLog("编辑客户")
    @SaCheckPermission("/biz/customer/edit")
    @PostMapping("/biz/customer/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizCustomerEditParam bizCustomerEditParam) {
        bizCustomerService.edit(bizCustomerEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除客户
     *
     * @author fanzherong
     * @date  2025/03/21 11:44
     */
    @Operation(summary = "删除客户")
    @CommonLog("删除客户")
    @SaCheckPermission("/biz/customer/delete")
    @PostMapping("/biz/customer/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizCustomerIdParam> bizCustomerIdParamList) {
        bizCustomerService.delete(bizCustomerIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取客户详情
     *
     * @author fanzherong
     * @date  2025/03/21 11:44
     */
    @Operation(summary = "获取客户详情")
    @GetMapping("/biz/customer/detail")
    public CommonResult<BizCustomer> detail(@Valid BizCustomerIdParam bizCustomerIdParam) {
        return CommonResult.data(bizCustomerService.detail(bizCustomerIdParam));
    }



    public static void main(String[] args) {
        // 获取时间戳单位S
        Long timesTamp = System.currentTimeMillis()/1000;
        System.out.println("time == " + timesTamp);
        Map<String, String> params = new ConcurrentHashMap<>(10);
        String appKey = "0ol2749zps9xhid1";
        String appSecret = "e4w9ylwk7grdsdzpbcq8kcedg7ailh1c";
        params.put("appKey",appKey);
        params.put("timestamp",String.valueOf(timesTamp));
        params.put("nonce","1241455595");// 自定义随机数（只能用一次）
        params.put("appSecret",appSecret);
        // 调用MD5算法加密生成签名
        String signature = generateSignature(params, appSecret);
        // 签名加入请求参数
        params.put("sign",signature);
        log.info("推送sign == " + signature);
        log.info("推送参数 == " + JSON.toJSONString(params));
        String result = HttpUtil.createPost("http://localhost:83/thirdPart/v1/customer/yongAdd").addHeaders(params).body("{}").execute().body();
        log.info("推送结果 == " + result);
    }


    public static String generateSignature(final Map<String, String> data, String key) {
        try {
            Set<String> keySet = data.keySet();
            String[] keyArray = keySet.toArray(new String[keySet.size()]);
            Arrays.sort(keyArray);
            StringBuilder sb = new StringBuilder();
            for (String k : keyArray) {
                if (k.equals("sign")) {
                    continue;
                }
                // 参数值为空，则不参与签名
                if (data.get(k).trim().length() > 0)  {
                    sb.append(k).append("=").append(data.get(k).trim()).append("&");
                }
            }
            System.out.printf(sb.toString());
            return MD5.md5(sb.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * 用友推送客户接口
     *
     * @author sandy
     * @date  2025/03/27 17:00
     */
    @Operation(summary = "用友推送客户接口")
    @CommonLog("用友推送客户接口")
    @PostMapping("/thirdPart/v1/customer/yongAdd")
    public CommonResult<String> yongAdd(@RequestBody @Valid BizCustomerYongAddParam bizCustomerYongAddParam) {
        bizCustomerService.addYong(bizCustomerYongAddParam);
        return CommonResult.ok();
    }

    /**
     * 获取用友平台销售订单
     *
     * @author sandy
     * @date  2025/03/27 17:00
     */
    @Operation(summary = "获取用友平台销售订单")
    @CommonLog("获取用友平台销售订单")
    @GetMapping("/biz/customer/getOrderByCustomerId")
    public CommonResult<Map<String, Object>> getOrderByCustomerId(BizCustomerIdParam bizCustomerIdParam){
        return CommonResult.data(bizCustomerService.getOrderByCustomerId(bizCustomerIdParam));
    }

}
